@@ -1,7 +1,7 @@
 # pylint: disable-msg=W0611
 #
-# Copyright (c) 2003 Sylvain Thenault (thenault@nerim.net)
-# Copyright (c) 2003-2004 Logilab
+# Copyright (c) 2003-2005 Sylvain Thenault (thenault@gmail.com)
+# Copyright (c) 2003-2005 Logilab (contact@logilab.fr)
 #
 # This program is free software; you can redistribute it and/or modify it under
 # the terms of the GNU General Public License as published by the Free Software
@@ -84,19 +84,26 @@ on From and Import :
  .get_module_object,
  
 """
+
 from __future__ import generators
 
 __author__ = "Sylvain Thenault"
-__revision__ = "$Id: astng.py,v 1.60 2005/01/04 10:51:28 syt Exp $"
+__revision__ = "$Id: astng.py,v 1.69 2005/02/15 13:41:48 syt Exp $"
 
+import __builtin__
 from compiler.ast import *
 from inspect import isclass
 
+import re
+ID_RGX = re.compile('^[a-zA-Z_][a-zA-Z_0-9]*$')
+del re
+
 from logilab.common.modutils import get_module_part, load_module_from_name
 from logilab.common.astng import ResolveError, NotFoundError, ASTNGManager, \
      ASTNGBuildingException
 from logilab.common.compat import chain        
 
+
 # Node  ######################################################################
 
 def is_statement(node):
@@ -162,6 +169,30 @@ def previous_sibling(node):
 
 Node.previous_sibling = previous_sibling
 
+def source_line(node):
+    """return the line number where the given node appears
+
+    we need this method since not all node as the lineno attribute
+    correctly set...
+    """
+    line = node.lineno
+    if line is None:
+        # FIXME: log node with the missing line no
+        _node = node
+        try:
+            while line is None:
+                _node = _node.getChildNodes()[0]
+                line = _node.lineno
+        except IndexError:
+            _node = node.parent
+            while _node and line is None:
+                line = _node.lineno
+                _node = _node.parent
+        node.lineno = line
+    return line
+
+Node.source_line = source_line
+
 def set_local(node, name, stmt):
     """define <name> in locals (<stmt> is the node defining the name)
     if the node is a Module node (i.e. has globals), add the name to globals
@@ -175,11 +206,25 @@ def set_local(node, name, stmt):
             
 Node.set_local = set_local
 
-import re
-ID_RGX = re.compile('^[a-zA-Z_][a-zA-Z_0-9]*$')
-del re
+def resolve_dotted(node, name):
+    """resolve a dotted names"""
+    parts = name.split('.')
+    baseobj = node.resolve(parts[0])
+    for part in parts[1:]:
+        baseobj = baseobj.resolve(part)
+    return baseobj
 
-import __builtin__
+Node.resolve_dotted = resolve_dotted
+
+def resolve_all(node, names):
+    """return an iterator the resolved names"""
+    for name in names:
+        try:
+            yield node.resolve_dotted(name)
+        except (ResolveError, NotFoundError, AssertionError):
+            continue
+        
+Node.resolve_all = resolve_all
 
 def resolve(node, name):
     """try to resolve the value (astng node) associated to an
@@ -189,6 +234,7 @@ def resolve(node, name):
 
     FIXME: refactor !!
     """
+    #print 'resolve', node.__class__, getattr(node, 'name', '?'), name
     assert ID_RGX.match(name), '%r is not a valid identifier' % name
     frame = node.get_frame()
     try:
@@ -215,74 +261,66 @@ def resolve(node, name):
                 raise ResolveError(name)
             # FIXME: may not be a class
             return ASTNGManager().astng_from_class(object, module)
-    # nodes with "object" attribute are final objects such as class / function
-    # definitions
-    if hasattr(stmt, 'object'):
-        return stmt
-    if isinstance(stmt, AssName):
-        assigned = stmt.get_assigned_value() or stmt
-        # FIXME: check name != assigned.name to avoid infinite loop
-        # if this is the case, we probably want to look for that name
-        # in globals/builtins, but this is not easlily feasible right now...
-        if isinstance(assigned, Name) and name != assigned.name:
-            return resolve(stmt.get_frame(), assigned.name)
-        return assigned
-        # FIXME
-    elif isinstance(stmt, Import) or isinstance(stmt, From):
-        context_file = node.root().file
-        try:
-            modname, obj = stmt.get_module_object(name, context_file)
-        # FIXME: log error...
-        except ImportError:
-            raise ResolveError(name)
-        except Exception, ex:
-            raise ResolveError(name)
-        # check that get_module_object seems to have worked correctly...
-        if obj is not None and obj.find('.') > -1:
-            raise ResolveError(name)
-        # handle special case where we are on a package node importing a module
-        # using the same name as the package, which may end in an infinite loop
-        # on relative imports
-        if modname == getattr(node, 'name', None):
-            # FIXME: I don't know what to do here...
-            raise ResolveError(name)
-        try:
-            module = ASTNGManager().astng_from_module_name(modname)
-        except (ASTNGBuildingException, SyntaxError):
-            # FIXME
-            raise ResolveError(name)
-            
-        assert not (module is node and obj == name), \
-               str((node.file, node.name, name))
-        if obj:
-            return module.resolve(obj)
-        return module
-    elif isinstance(stmt, Global):
-        return stmt.root().resolve(name)
-    return stmt
+    return stmt.self_resolve(name)
     
 Node.resolve = resolve
 
-def resolve_dotted(node, name):
-    """resolve a dotted names"""
-    parts = name.split('.')
-    baseobj = node.resolve(parts[0])
-    for part in parts[1:]:
-        baseobj = baseobj.resolve(part)
-    return baseobj
+def self_resolve(node, name):
+    """self resolve return the node itself by default"""
+    return node
+Node.self_resolve = self_resolve
 
-Node.resolve_dotted = resolve_dotted
+def assname_self_resolve(node, name):
+    """self resolve on AssName try to return the assigned value"""
+    # FIXME
+    assigned = node.get_assigned_value() or node
+    # FIXME: check name != assigned.name to avoid infinite loop
+    # if this is the case, we probably want to look for that name
+    # in globals/builtins, but this is not easlily feasible right now...
+    if isinstance(assigned, Name) and name != assigned.name:
+        return resolve(node.get_frame(), assigned.name)
+    return assigned
+AssName.self_resolve = assname_self_resolve
 
+def import_self_resolve(node, name):
+    """self resolve on From / Import nodes return the imported module/object"""
+    context_file = node.root().file
+    try:
+        modname, obj = node.get_module_object(name, context_file)
+    # FIXME: log error...
+    except ImportError:
+        raise ResolveError(name)
+    except Exception, ex:
+        #import traceback
+        #traceback.print_exc()
+        raise ResolveError(name)
+    # check that get_module_object seems to have worked correctly...
+    if obj is not None and obj.find('.') > -1:
+        raise ResolveError(name)
+    # handle special case where we are on a package node importing a module
+    # using the same name as the package, which may end in an infinite loop
+    # on relative imports
+    if modname == getattr(node.root(), 'name', None):
+        # FIXME: I don't know what to do here...
+        raise ResolveError(name)
+    try:
+        module = ASTNGManager().astng_from_module_name(modname, context_file)
+    except (ASTNGBuildingException, SyntaxError):
+        # FIXME
+        raise ResolveError(name)
 
-def resolve_all(node, names):
-    """return an iterator the resolved names"""
-    for name in names:
-        try:
-            yield node.resolve_dotted(name)
-        except (ResolveError, NotFoundError):
-            continue
+    assert not (module is node and obj == name), \
+           str((node.file, node.name, name))
+    if obj:
+        return module.resolve(obj)
+    return module
+Import.self_resolve = import_self_resolve
+From.self_resolve = import_self_resolve
 
-Node.resolve_all = resolve_all
+def global_self_resolve(node, name):
+    """self resolve on Global is equivalent to resolve at the module level"""
+    return node.root().resolve(name)
+Global.self_resolve = global_self_resolve
 
 
 # module class dict/iterator interface ########################################
@@ -310,6 +348,7 @@ def module_values(node):
     return node.locals.values()
 def module_items(node):
     return node.locals.items()
+
 Module.__getitem__ = node_getitem
 Module.__iter__ = node_iter
 Module.keys = node_keys
@@ -332,12 +371,14 @@ def module_append_node(node, child_node):
     """append a child to the given node"""
     node.node.nodes.append(child_node)
     child_node.parent = node
+    
 Module.append_node = module_append_node
 
 def base_append_node(node, child_node):
     """append a child which should alter locals to the given node"""
     node.code.nodes.append(child_node)
     child_node.parent = node
+    
 Class.append_node = base_append_node
 Function.append_node = base_append_node
 
@@ -345,6 +386,7 @@ def add_local_node(node, child_node):
     """append a child which should alter locals to the given node"""
     node.append_node(child_node)
     node.set_local(child_node.name, child_node)
+    
 Module.add_local_node = add_local_node
 Function.add_local_node = add_local_node
 Class.add_local_node = add_local_node
@@ -356,6 +398,7 @@ def class_set_parents(node, basenames):
         base.parent = node
     node.basenames = basenames
     node.bases = bases
+    
 Class.set_parents = class_set_parents
 
 # Module  #####################################################################
@@ -366,8 +409,26 @@ def module_get_statement(node):
 
 Module.get_statement = module_get_statement
 
+def module_wildcard_import_names(node):
+    """return the list of imported names when this module is 'wildard imported'
+
+    FIXME: should i include '__builtins__' which is added by the actual
+           cpython implementation ?
+    """
+    try:
+        explicit = node.locals['__all__'].get_assigned_value()
+    except KeyError:
+        return [n for n in node.locals.keys() if not n.startswith('_')]
+    else:
+        # should be a tuple of constant string
+        return [const.value for const in explicit.nodes]
+
+Module.wildcard_import_names = module_wildcard_import_names
+
 # Function  ###################################################################
 
+Function.type = None # 'staticmethod' / 'classmethod'
+
 def is_method(node):
     """return true if the function should be considered as a method"""
     return isinstance(node.parent.get_frame(), Class)
@@ -451,6 +512,9 @@ def get_ancestor_for_attribute(node, attr):
     for astng in ancestors(node):
         if astng.instance_attrs.has_key(attr):
             return astng
+#    anode = get_ancestor_for_class_attribute(node, attr)
+#    if not isinstance(anode, Function):
+#        return anode
     raise NotFoundError(attr)
 
 Class.get_ancestor_for_attribute = get_ancestor_for_attribute
@@ -997,4 +1061,3 @@ def _import_string(names):
         else:
             _names.append(name)
     return  ', '.join(_names)
-
