@@ -16,7 +16,7 @@ Copyright (c) 2003-2004 LOGILAB S.A. (Paris, FRANCE).
 http://www.logilab.fr/ -- mailto:contact@logilab.fr
 """
 
-__revision__ = "$Id: unittest_astng.py,v 1.17 2005/01/04 10:51:28 syt Exp $"
+__revision__ = "$Id: unittest_astng.py,v 1.20 2005/02/15 13:40:50 syt Exp $"
 
 import unittest
 import sys
@@ -24,16 +24,29 @@ import parser
 from os import getcwd
 from os.path import join
 
-from logilab.common.astng import builder, astng, ResolveError, NotFoundError
+from logilab.common.astng import builder, astng, ResolveError, NotFoundError, \
+     ASTNGManager
 from logilab.common.modutils import load_module_from_name
 
 import data
 from data import module as test_module
 
-MODULE = builder.ASTNGBuilder().build_from_module(test_module)
-MODULE2 = builder.ASTNGBuilder().file_build('data/module2.py', 'data.module2')
-NONREGR = builder.ASTNGBuilder().file_build('data/nonregr.py', 'data.nonregr')
+manager = ASTNGManager()
+abuilder = builder.ASTNGBuilder(manager) 
+MODULE = abuilder.build_from_module(test_module)
+MODULE2 = abuilder.file_build('data/module2.py', 'data.module2')
+NONREGR = abuilder.file_build('data/nonregr.py', 'data.nonregr')
 
+class ModuleTC(unittest.TestCase):
+    
+    def test_wildard_import_names(self):
+        m = abuilder.file_build('astng_data/all.py', 'all')
+        self.assertEquals(m.wildcard_import_names(), ['Aaa', '_bla', 'name'])
+        m = abuilder.file_build('astng_data/notall.py', 'notall')
+        res = m.wildcard_import_names()
+        res.sort()
+        self.assertEquals(res, ['Aaa', 'func', 'name', 'other'])
+        
 class ASTNGRepresentationTest(unittest.TestCase):
 
     def test_module_base_props(self):
@@ -56,10 +69,10 @@ class ASTNGRepresentationTest(unittest.TestCase):
         keys = _locals.keys()
         keys.sort()
         self.assert_(_locals is MODULE.globals)
-        self.assertEquals(len(_locals), 14)
+        self.assertEquals(len(_locals), 13)
         # __builtins__ is introduced by "from toto import *"
-        self.assertEquals(keys, ['MY_DICT', 'YO', 'YOUPI', '__builtins__',
-                                '__revision__',
+        self.assertEquals(keys, ['MY_DICT', 'YO', 'YOUPI',
+                                 '__revision__',
                                 'clean', 'cvrtr', 'debuild', 'global_access',
                                 'modutils', 'nested_args', 'os', 'redirect', 'spawn'])
 
@@ -258,7 +271,7 @@ class ASTNGRepresentationTest(unittest.TestCase):
 
     def test_build_from_living(self):
         import time
-        timeastng = builder.ASTNGBuilder().build_from_module(time)
+        timeastng = abuilder.build_from_module(time)
         self.assert_(timeastng)
 
     def test_resolve_astng(self):
@@ -313,9 +326,9 @@ class ASTNGRepresentationTest(unittest.TestCase):
         _object = yo.resolve('object')
         self.assert_(isinstance(_object, astng.Class))
         self.assertEquals(_object.name, 'object')
-        # check builtin function has argnames == []
+        # check builtin function has argnames == None
         _object = yo.resolve('dict')
-        self.assertEquals(_object.locals['has_key'].argnames, [])
+        self.assertEquals(_object.locals['has_key'].argnames, None)
 
     def test_resolve_raise(self):
         self.assertRaises(ResolveError, MODULE.resolve, 'YOAA')
@@ -324,6 +337,17 @@ class ASTNGRepresentationTest(unittest.TestCase):
         self.assertEquals(NONREGR.resolve('enumerate').name, 'enumerate')
         self.assertRaises(ResolveError, NONREGR.locals['toto'].resolve_dotted, 'v.get')
                           
+    def test_resolve_package_redirection(self):
+        sys.path.insert(1, 'astng_data')
+        try:
+            m = abuilder.file_build('astng_data/appl/myConnection.py', 'appl.myConnection')
+            cnx = m.resolve_dotted('SSL1.Connection')
+            self.assertEquals(cnx.__class__, astng.Class)
+            self.assertEquals(cnx.name, 'Connection')
+            self.assertEquals(cnx.root().name, 'Connection1')
+        finally:
+            del sys.path[1]
+        
     def test_get_assigned_value(self):
         my_dict = MODULE.locals['MY_DICT']
         self.assert_(isinstance(my_dict.get_assigned_value(), astng.Dict))
@@ -400,7 +424,7 @@ class UtilsTC(unittest.TestCase):
 class BuilderTC(unittest.TestCase):
     def test_noendingnewline(self):
         """check that a file with no trailing new line is parseable"""
-        builder.ASTNGBuilder().file_build('data/noendingnewline.py', 'data.noendingnewline')
+        abuilder.file_build('data/noendingnewline.py', 'data.noendingnewline')
         
 if __name__ == '__main__':
     unittest.main()
